@@ -1,5 +1,6 @@
 #include "gcin.h"
 #include "gtab.h"
+#include <signal.h>
 #include <X11/extensions/XTest.h>
 
 
@@ -11,7 +12,7 @@ extern XIMS current_ims;
 static IMForwardEventStruct *current_forward_eve;
 #endif
 
-static char callback_str_buffer[32];
+static char *callback_str_buffer;
 Window focus_win;
 static int timeout_handle;
 
@@ -24,31 +25,35 @@ static void send_fake_key_eve()
 
 void send_text_call_back(char *text)
 {
+  callback_str_buffer = realloc(callback_str_buffer, strlen(text));
   strcpy(callback_str_buffer, text);
   send_fake_key_eve();
 }
 
 
 ClientState *current_CS;
-ClientState temp_CS;
+static ClientState temp_CS;
 
 gboolean init_in_method(int in_no);
 
-char *output_buffer, *output_buffer_bak;
-int  output_bufferN;
+char *output_buffer;
+int output_bufferN;
+static char *output_buffer_raw, *output_buffer_raw_bak;
+static int output_buffer_rawN;
 
 void clear_output_buffer()
 {
-  char *t;
-
-  t = output_buffer;
-  output_buffer = output_buffer_bak;
-  output_buffer_bak = t;
-
   if (output_buffer)
     output_buffer[0] = 0;
-
   output_bufferN = 0;
+
+  char *t = output_buffer_raw;
+  output_buffer_raw = output_buffer_raw_bak;
+  output_buffer_raw_bak = t;
+
+  if (output_buffer_raw)
+    output_buffer_raw[0] = 0;
+  output_buffer_rawN = 0;
 }
 
 gboolean gb_output = FALSE;
@@ -58,6 +63,14 @@ void toggle_gb_output()
   gb_output = !gb_output;
 }
 
+static void append_str(char **buf, int *bufN, char *text, int len)
+{
+  int requiredN = len + 1 + *bufN;
+  *buf = realloc(*buf, requiredN);
+  (*buf)[*bufN] = 0;
+  strcat(*buf, text);
+  *bufN += len;
+}
 
 void send_text(char *text)
 {
@@ -65,6 +78,8 @@ void send_text(char *text)
     return;
   int len = strlen(text);
 
+  append_str(&output_buffer_raw, &output_buffer_rawN, text, len);
+
   char *utf8_gbtext = NULL;
 
   if (gb_output) {
@@ -99,20 +114,63 @@ void send_text(char *text)
     len = wn;
   }
 
-  int requiredN = len + 1 + output_bufferN;
+  char *filter = getenv("GCIN_OUTPUT_FILTER");
+  char filter_text[512];
 
-  output_buffer = realloc(output_buffer, requiredN);
-  output_buffer[output_bufferN] = 0;
+  if (filter) {
+    int pfdr[2], pfdw[2];
 
-  strcat(output_buffer, text);
-  output_bufferN += len;
+    if (pipe(pfdr) == -1) {
+      dbg("cannot pipe r\n");
+      goto next;
+    }
+
+    if (pipe(pfdw) == -1) {
+      dbg("cannot pipe w\n");
+      goto next;
+    }
+
+    int pid = fork();
+
+    if (pid < 0) {
+      dbg("cannot fork filter\n");
+      goto next;
+    }
+
+    if (pid) {
+      close(pfdw[0]);
+      close(pfdr[1]);
+      write(pfdw[1], text, len);
+      close(pfdw[1]);
+      int rn = read(pfdr[0], filter_text, sizeof(filter_text) - 1);
+      filter_text[rn] = 0;
+//      puts(filter_text);
+      close(pfdr[0]);
+      text = filter_text;
+      len = rn;
+    } else {
+      close(pfdr[0]);
+      close(pfdw[1]);
+      dup2(pfdw[0], 0);
+      dup2(pfdr[1], 1);
+      if (execl(filter, filter, NULL) < 0) {
+        dbg("execl %s err", filter);
+        goto next;
+      }
+    }
+
+  }
+
+next:
+  if (len)
+    append_str(&output_buffer, &output_bufferN, text, len);
 
   g_free(utf8_gbtext);
 }
 
 void send_output_buffer_bak()
 {
-  send_text(output_buffer_bak);
+  send_text(output_buffer_raw_bak);
 }
 
 void sendkey_b5(char *bchar)
@@ -134,7 +192,6 @@ void export_text_xim()
   if (!output_bufferN)
     return;
 
-
   XTextProperty tp;
   char outbuf[512];
 
@@ -647,6 +704,7 @@ static KeySym last_keysym;
 
 gboolean timeout_raise_window()
 {
+//  dbg("timeout_raise_window\n");
   timeout_handle = 0;
   show_in_win(current_CS);
   return FALSE;
@@ -664,7 +722,7 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
   if (current_CS->client_win)
     focus_win = current_CS->client_win;
 
-  if (strlen(callback_str_buffer)) {
+  if (callback_str_buffer && strlen(callback_str_buffer)) {
     send_text(callback_str_buffer);
     callback_str_buffer[0]=0;
     return TRUE;
@@ -813,8 +871,6 @@ int xim_ForwardEventHandler(IMForwardEventStruct *call_data)
 }
 #endif
 
-IC *FindIC(CARD16 icid);
-void load_IC(IC *rec);
 
 int gcin_FocusIn(ClientState *cs)
 {
@@ -861,6 +917,8 @@ int gcin_FocusIn(ClientState *cs)
 
 
 #if USE_XIM
+IC *FindIC(CARD16 icid);
+void load_IC(IC *rec);
 int xim_gcin_FocusIn(IMChangeFocusStruct *call_data)
 {
     IC *ic = FindIC(call_data->icid);
@@ -879,18 +937,39 @@ int xim_gcin_FocusIn(IMChangeFocusStruct *call_data)
 }
 #endif
 
+extern Window xwin_pho, xwin0, xwin_gtab;
+
+gint64 current_time();
+
+static gint64 last_focus_out_time;
+static ClientState *last_cs;
+
 int gcin_FocusOut(ClientState *cs)
 {
-    if (cs == current_CS) {
-      hide_in_win(cs);
-    }
+  gint64 t = current_time();
 
-    reset_current_in_win_xy();
+  if (t - last_focus_out_time < 100000 && cs == last_cs) {
+    last_focus_out_time = t;
+    return;
+  }
 
-    if (cs == current_CS)
-      temp_CS = *current_CS;
+  last_focus_out_time = t;
+  last_cs = cs;
 
-    return True;
+  if (cs == current_CS) {
+    hide_in_win(cs);
+  }
+
+  reset_current_in_win_xy();
+
+  if (cs == current_CS)
+    temp_CS = *current_CS;
+
+#if 0
+  dbg("focus out\n");
+#endif
+
+  return True;
 }
 
 #if USE_XIM
