@@ -13,6 +13,9 @@ static struct {
  {"倚天 26 鍵", "et26"},
  {"倚天 26 鍵,使用 asdf 選擇", "et26-asdf"},
  {"許氏(國音,自然)", "hsu"},
+ {"聲調拼音", "pinyin"},
+ {"聲調拼音, 使用 asdf 選擇", "pinyin-asdf"},
+{"Dvorak", "dvorak"},
  {"IBM", "ibm"},
  {NULL, NULL}
 };
@@ -24,6 +27,7 @@ static GtkWidget *check_button_tsin_phrase_pre_select,
                  *check_button_pho_in_row1,
                  *check_button_phonetic_huge_tab,
                  *check_button_tsin_tone_char_input,
+                 *check_button_tsin_tab_phrase_end,
                  *spinner_tsin_buffer_size;
 
 static GtkWidget *opt_kbm_opts;
@@ -93,6 +97,10 @@ static gboolean cb_ok( GtkWidget *widget,
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_tsin_tone_char_input)));
 
 
+  save_gcin_conf_int(TSIN_TAB_PHRASE_END,
+       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_tsin_tab_phrase_end)));
+
+
   tsin_buffer_size = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinner_tsin_buffer_size));
   save_gcin_conf_int(TSIN_BUFFER_SIZE, tsin_buffer_size);
 
@@ -130,15 +138,16 @@ static void callback_button_clicked_tsin_space_opt( GtkWidget *widget, gpointer
 static int get_current_kbm_idx()
 {
   char kbm_str[32];
-  get_gcin_conf_str("phonetic-keyboard", kbm_str, "zo");
+  get_gcin_conf_fstr(PHONETIC_KEYBOARD, kbm_str, "zo-asdf");
 
-  int i;
+  int i, rval;
   for(i=0; kbm_sel[i].kbm; i++)
-    if (!strcmp(kbm_sel[i].kbm, kbm_str))
+    if (!strcmp(kbm_sel[i].kbm, kbm_str)) {
       return i;
+    }
 
   p_err("phonetic-keyboard->%s is not valid", kbm_str);
-  return -1;
+  return 0;
 }
 
 static int get_currnet_eng_ch_sw_idx()
@@ -431,6 +440,16 @@ void create_kbm_window()
      GTK_TOGGLE_BUTTON(check_button_tsin_tone_char_input), tsin_tone_char_input);
 
 
+  GtkWidget *hbox_tsin_tab_phrase_end = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox_top), hbox_tsin_tab_phrase_end , TRUE, TRUE, 1);
+  GtkWidget *label_tsin_tab_phrase_end = gtk_label_new("詞音 Tab 斷詞");
+  gtk_box_pack_start (GTK_BOX (hbox_tsin_tab_phrase_end), label_tsin_tab_phrase_end , TRUE, TRUE, 0);
+  check_button_tsin_tab_phrase_end = gtk_check_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox_tsin_tab_phrase_end), check_button_tsin_tab_phrase_end, FALSE, FALSE, 0);
+  gtk_toggle_button_set_active(
+     GTK_TOGGLE_BUTTON(check_button_tsin_tab_phrase_end), tsin_tab_phrase_end);
+
+
   GtkWidget *frame_tsin_buffer_size = gtk_frame_new("詞音編輯緩衝區大小");
   gtk_box_pack_start (GTK_BOX (vbox_top), frame_tsin_buffer_size, FALSE, FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (frame_tsin_buffer_size), 1);
