@@ -14,10 +14,13 @@ static GtkWidget *check_button_gtab_dup_select_bell,
                  *check_button_gtab_shift_phrase_key,
                  *check_button_gtab_hide_row2,
                  *check_button_gtab_in_row1,
+                 *check_button_gtab_capslock_in_eng,
                  *check_button_root_style_use,
                  *check_button_gcin_pop_up_win,
-                 *check_button_gcin_pop_up_win_abs_corner,
                  *check_button_gcin_inner_frame;
+#if TRAY_ENABLED
+                 *check_button_gcin_status_tray;
+#endif
 
 static GtkWidget *opt_spc_opts;
 
@@ -82,7 +85,7 @@ static void cb_file_ts_export(GtkWidget *widget, gpointer user_data)
    get_gcin_dir(gcin_dir);
 
    char cmd[256];
-   snprintf(cmd, sizeof(cmd), GCIN_BIN_DIR"/tsd2a %s/tsin > %s", gcin_dir, selected_filename);
+   snprintf(cmd, sizeof(cmd), GCIN_BIN_DIR"/tsd2a32 %s/tsin32 > %s", gcin_dir, selected_filename);
    int res = system(cmd);
    create_result_win(res);
 }
@@ -121,7 +124,7 @@ static void cb_file_ts_import(GtkWidget *widget, gpointer user_data)
 
    char cmd[256];
    snprintf(cmd, sizeof(cmd),
-      "cd %s/.gcin && "GCIN_BIN_DIR"/tsd2a tsin > tmpfile && cat %s >> tmpfile && "GCIN_BIN_DIR"/tsa2d tmpfile",
+      "cd %s/.gcin && "GCIN_BIN_DIR"/tsd2a32 tsin32 > tmpfile && cat %s >> tmpfile && "GCIN_BIN_DIR"/tsa2d32 tmpfile",
       getenv("HOME"), selected_filename);
    int res = system(cmd);
    create_result_win(res);
@@ -157,7 +160,7 @@ static void cb_ts_edit()
 {
   char tt[512];
 
-  sprintf(tt, "( cd ~/.gcin && "GCIN_BIN_DIR"/tsd2a tsin > tmpfile && %s tmpfile && "GCIN_BIN_DIR"/tsa2d tmpfile ) &", utf8_edit);
+  sprintf(tt, "( cd ~/.gcin && "GCIN_BIN_DIR"/tsd2a32 tsin32 > tmpfile && %s tmpfile && "GCIN_BIN_DIR"/tsa2d32 tmpfile ) &", utf8_edit);
   dbg("exec %s\n", tt);
   system(tt);
 }
@@ -167,7 +170,7 @@ static void cb_ts_import_sys()
 {
   char tt[512];
 
-  sprintf(tt, "cd ~/.gcin && "GCIN_BIN_DIR"/tsd2a tsin > tmpfile && "GCIN_BIN_DIR"/tsd2a %s/tsin >> tmpfile && "GCIN_BIN_DIR"/tsa2d tmpfile", GCIN_TABLE_DIR);
+  sprintf(tt, "cd ~/.gcin && "GCIN_BIN_DIR"/tsd2a32 tsin32 > tmpfile && "GCIN_BIN_DIR"/tsd2a32 %s/tsin32 >> tmpfile && "GCIN_BIN_DIR"/tsa2d32 tmpfile", GCIN_TABLE_DIR);
   dbg("exec %s\n", tt);
   system(tt);
 }
@@ -264,9 +267,6 @@ static gboolean cb_appearance_conf_ok( GtkWidget *widget,
   int gcin_pop_up_win = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_pop_up_win));
   save_gcin_conf_int(GCIN_POP_UP_WIN, gcin_pop_up_win);
 
-  int gcin_pop_up_win_abs_corner = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_pop_up_win_abs_corner));
-  save_gcin_conf_int(GCIN_POP_UP_WIN_ABS_CORNER, gcin_pop_up_win_abs_corner);
-
   int gcin_root_x = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinner_root_style_x));
   save_gcin_conf_int(GCIN_ROOT_X, gcin_root_x);
 
@@ -278,7 +278,9 @@ static gboolean cb_appearance_conf_ok( GtkWidget *widget,
   save_gcin_conf_int(GCIN_INPUT_STYLE, style);
 
   save_gcin_conf_int(GCIN_INNER_FRAME, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_inner_frame)));
-
+#if TRAY_ENABLED
+  save_gcin_conf_int(GCIN_STATUS_TRAY, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_status_tray)));
+#endif
 
 
   send_gcin_message(GDK_DISPLAY(), CHANGE_FONT_SIZE);
@@ -383,32 +385,15 @@ void create_appearance_conf_window()
   gtk_box_pack_start (GTK_BOX (vbox_top), font_sel, FALSE, FALSE, 0);
 #endif
 
-  GtkWidget *frame_gcin_pop_up_win = gtk_frame_new("彈出式輸入視窗");
-  gtk_box_pack_start (GTK_BOX(vbox_top), frame_gcin_pop_up_win, FALSE, FALSE, 0);
-
-  GtkWidget *hbox_gcin_pop_up_win0 = gtk_hbox_new (TRUE, 10);
-  gtk_container_add (GTK_CONTAINER (frame_gcin_pop_up_win), hbox_gcin_pop_up_win0);
-
   GtkWidget *hbox_gcin_pop_up_win = gtk_hbox_new (FALSE, 10);
-  gtk_box_pack_start (GTK_BOX(hbox_gcin_pop_up_win0), hbox_gcin_pop_up_win, FALSE, FALSE, 0);
-  GtkWidget *label_gcin_pop_up_win = gtk_label_new("使用");
+  gtk_box_pack_start (GTK_BOX(vbox_top), hbox_gcin_pop_up_win, FALSE, FALSE, 0);
+  GtkWidget *label_gcin_pop_up_win = gtk_label_new("彈出式輸入視窗");
   gtk_box_pack_start (GTK_BOX(hbox_gcin_pop_up_win), label_gcin_pop_up_win, FALSE, FALSE, 0);
   check_button_gcin_pop_up_win = gtk_check_button_new ();
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_pop_up_win),
        gcin_pop_up_win);
   gtk_box_pack_start (GTK_BOX(hbox_gcin_pop_up_win), check_button_gcin_pop_up_win, FALSE, FALSE, 0);
 
-
-  GtkWidget *hbox_gcin_pop_up_win_abs_corner = gtk_hbox_new (FALSE, 10);
-  gtk_box_pack_start (GTK_BOX(hbox_gcin_pop_up_win0), hbox_gcin_pop_up_win_abs_corner, FALSE, FALSE, 0);
-  GtkWidget *label_gcin_pop_up_win_abs_corner = gtk_label_new("絕對右下角");
-  gtk_box_pack_start (GTK_BOX(hbox_gcin_pop_up_win_abs_corner), label_gcin_pop_up_win_abs_corner, FALSE, FALSE, 0);
-  check_button_gcin_pop_up_win_abs_corner = gtk_check_button_new ();
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_pop_up_win_abs_corner),
-       gcin_pop_up_win_abs_corner);
-  gtk_box_pack_start (GTK_BOX(hbox_gcin_pop_up_win_abs_corner), check_button_gcin_pop_up_win_abs_corner, FALSE, FALSE, 0);
-
-
   GtkWidget *frame_root_style = gtk_frame_new("固定 gcin 視窗位置");
   gtk_box_pack_start (GTK_BOX (vbox_top), frame_root_style, FALSE, FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (frame_root_style), 3);
@@ -448,6 +433,16 @@ void create_appearance_conf_window()
        gcin_inner_frame);
   gtk_box_pack_start (GTK_BOX(hbox_gcin_inner_frame), check_button_gcin_inner_frame, FALSE, FALSE, 0);
 
+#if TRAY_ENABLED
+  GtkWidget *hbox_gcin_status_tray = gtk_hbox_new (FALSE, 10);
+  gtk_box_pack_start (GTK_BOX(vbox_top), hbox_gcin_status_tray, FALSE, FALSE, 0);
+  GtkWidget *label_gcin_status_tray = gtk_label_new("面板狀態(tray)");
+  gtk_box_pack_start (GTK_BOX(hbox_gcin_status_tray), label_gcin_status_tray, FALSE, FALSE, 0);
+  check_button_gcin_status_tray = gtk_check_button_new ();
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_status_tray),
+       gcin_status_tray);
+  gtk_box_pack_start (GTK_BOX(hbox_gcin_status_tray), check_button_gcin_status_tray, FALSE, FALSE, 0);
+#endif
 
 
   GtkWidget *hbox_cancel_ok = gtk_hbox_new (FALSE, 10);
@@ -517,6 +512,10 @@ static gboolean cb_gtab_conf_ok( GtkWidget *widget,
   save_gcin_conf_int(GTAB_IN_ROW1,
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_in_row1)));
 
+  save_gcin_conf_int(GTAB_CAPSLOCK_IN_ENG,
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_capslock_in_eng)));
+
+
   int idx = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_spc_opts));
   save_gcin_conf_int(GTAB_SPACE_AUTO_FIRST, spc_opts[idx].num);
 
@@ -727,6 +726,17 @@ void create_gtab_conf_window()
        gtab_in_row1);
 
 
+    GtkWidget *hbox_gtab_capslock_in_eng = gtk_hbox_new (FALSE, 10);
+    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_capslock_in_eng, FALSE, FALSE, 0);
+    GtkWidget *label_gtab_capslock_in_eng = gtk_label_new("CapsLock 打開輸入英數");
+    gtk_box_pack_start (GTK_BOX (hbox_gtab_capslock_in_eng), label_gtab_capslock_in_eng,  FALSE, FALSE, 0);
+    check_button_gtab_capslock_in_eng = gtk_check_button_new ();
+    gtk_box_pack_start (GTK_BOX (hbox_gtab_capslock_in_eng), check_button_gtab_capslock_in_eng,  FALSE, FALSE, 0);
+
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_capslock_in_eng),
+       gtab_capslock_in_eng);
+
+
     GtkWidget *hbox_cancel_ok = gtk_hbox_new (FALSE, 10);
     gtk_box_pack_start (GTK_BOX (vbox_top), hbox_cancel_ok, FALSE, FALSE, 0);
 
@@ -777,12 +787,14 @@ static void cb_gb_output_toggle()
 static void cb_gb_translate_toggle()
 {
   system(GCIN_BIN_DIR"/sim2trad &");
+  exit(0);
 }
 
 
 static void cb_juying_learn()
 {
-  system(GCIN_BIN_DIR"/juyin-learn&");
+  system(GCIN_BIN_DIR"/juyin-learn &");
+  exit(0);
 }
 
 
