@@ -146,7 +146,6 @@ int xim_gcin_FocusOut(IMChangeFocusStruct *call_data);
 
 #define DEBUG 0
 
-
 int gcin_ProtoHandler(XIMS ims, IMProtocol *call_data)
 {
 //  dbg("gcin_ProtoHandler %x ims\n", ims);
@@ -292,7 +291,8 @@ void open_xim()
             IMOnKeysList, &triggerKeys,
   //        IMOffKeysList, &triggerKeys,
             NULL)) == NULL) {
-            p_err("IMOpenIM failed. Maybe another XIM server is running.\n");
+            p_err("IMOpenIM '%s' failed. Maybe another XIM server is running.\n",
+            xim_arr[i].xim_server_name);
     }
   }
 }
@@ -343,13 +343,16 @@ static int xerror_handler(Display *d, XErrorEvent *eve)
 
 Atom gcin_atom;
 void cb_trad_sim_toggle();
+void execute_message(char *message);
 
 static GdkFilterReturn my_gdk_filter(GdkXEvent *xevent,
                                      GdkEvent *event,
                                      gpointer data)
 {
    XEvent *xeve = (XEvent *)xevent;
-
+#if 0
+   dbg("a zzz %d\n", xeve->type);
+#endif
    if (xeve->type == PropertyNotify) {
      XPropertyEvent *xprop = &xeve->xproperty;
 
@@ -374,16 +377,30 @@ static GdkFilterReturn my_gdk_filter(GdkXEvent *xevent,
        if (!strcmp(message, GB_OUTPUT_TOGGLE)) {
          cb_trad_sim_toggle();
        } else
+       if (strstr(message, "#gcin_message")) {
+         execute_message(message);
+       }
+       else
          reload_data();
 
        XFree(message);
        return GDK_FILTER_REMOVE;
      }
    }
+
+   // only very old WM will enter this
+   if (xeve->type == FocusIn || xeve->type == FocusOut) {
+#if 0
+     dbg("focus %s\n", xeve->type == FocusIn ? "in":"out");
+#endif
+     return GDK_FILTER_REMOVE;
+   }
+
 #if USE_XIM
    if (XFilterEvent(xeve, None) == True)
      return GDK_FILTER_REMOVE;
 #endif
+
    return GDK_FILTER_CONTINUE;
 }
 
@@ -397,8 +414,8 @@ void init_atom_property()
 
 void hide_win0();
 void do_exit();
-void destory_win0();
-void destory_win1();
+void destroy_win0();
+void destroy_win1();
 void destroy_win_gtab();
 void free_pho_mem(),free_tsin(),free_all_IC(), free_gtab(), free_phrase();
 
@@ -415,8 +432,8 @@ void do_exit()
   free_phrase();
 
 #if 1
-  destory_win0();
-  destory_win1();
+  destroy_win0();
+  destroy_win1();
   destroy_win_gtab();
 #endif
 
@@ -461,6 +478,9 @@ int main(int argc, char **argv)
   char *lc_utf8;
   char *lc;
 
+  signal(SIGCHLD, SIG_IGN);
+  signal(SIGPIPE, SIG_IGN);
+
   if (lc_ctype && strstr(lc_ctype, "en") /* || lang && strstr(lang, "en") */){
     lc = "en_US";
     enc = "UTF-8";
