@@ -25,7 +25,8 @@ extern char tsidxfname[64];
 extern int hashidx[TSIN_HASH_N];
 
 CHPHO chpho[MAX_PH_BF_EXT];
-int c_idx, c_len, ph_sta=-1, ph_sta_last=-1;
+int c_idx, c_len;
+int ph_sta=-1, ph_sta_last=-1;  // phrase start
 static int sel_pho;
 static gboolean eng_ph=TRUE;  // english(FALSE) <-> pho(juyin, TRUE)
 static int save_frm, save_to;
@@ -96,14 +97,14 @@ void drawcursor()
   last_cursor_idx = c_idx;
 
   if (c_idx == c_len) {
-    if (tsin_half_full || eng_ph) {
-#if 0
-      disp_char(c_idx,"  ");
-      set_cursor_tsin(c_idx);
-#endif
-    } else {
-      disp_char(c_idx, " ");
-      set_cursor_tsin(c_idx);
+    if (!eng_ph) {
+      if (tsin_half_full) {
+        disp_char(c_idx,"  ");
+        set_cursor_tsin(c_idx);
+      } else {
+        disp_char(c_idx, " ");
+        set_cursor_tsin(c_idx);
+      }
     }
   }
   else {
@@ -175,8 +176,11 @@ static void putbuf(int len)
 
       chpho_extract(&chpho[i], phrlen, pho, ch, och);
 
-      if (chpho[i].flag & FLAG_CHPHO_PHRASE_VOID)
+      if (chpho[i].flag & FLAG_CHPHO_PHRASE_VOID) {
+#if 0
         inc_dec_tsin_use_count(pho, och, phrlen, TRUE);
+#endif
+      }
       else
         inc_dec_tsin_use_count(pho, ch, phrlen, FALSE);
     }
@@ -225,15 +229,20 @@ static void prbuf()
 }
 
 
-static int orig_disp_in_area_x = -1;
 void disp_tsin_pho(int index, char *pho);
 
 static void disp_in_area_pho_tsin()
 {
   int i;
 
-  for(i=0;i<4;i++) {
-    disp_tsin_pho(i, &pho_chars[i][typ_pho[i] * PHO_CHAR_LEN]);
+  if (pin_juyin) {
+    for(i=0;i<6;i++) {
+      disp_tsin_pho(i, &inph[i]);
+    }
+  } else {
+    for(i=0;i<4;i++) {
+      disp_tsin_pho(i, &pho_chars[i][typ_pho[i] * PHO_CHAR_LEN]);
+    }
   }
 }
 
@@ -447,6 +456,14 @@ static void save_phrase()
   return;
 }
 
+
+static void set_fixed(int idx, int len)
+{
+  int i;
+  for(i=idx; i < idx+len; i++)
+    chpho[i].flag |= FLAG_CHPHO_FIXED;
+}
+
 #define PH_SHIFT_N (tsin_buffer_size - 1)
 
 static void shift_ins()
@@ -460,6 +477,12 @@ static void shift_ins()
    if (c_len >= PH_SHIFT_N) {
      int ofs;
 
+     // set it fixed so that it will not cause partial phrase in the beginning
+     int fixedlen = c_len - 10;
+     if (fixedlen <= 0)
+       fixedlen = 1;
+     set_fixed(0, Min(c_len, fixedlen));
+
      ofs = 1;
      putbuf(ofs);
 
@@ -478,7 +501,7 @@ static void shift_ins()
        chpho[j+1] = chpho[j];
      }
 
-     chpho[c_len].ch[0]=' ';
+     init_chpho_i(c_len);
     /*    prbuf(); */
    }
 
@@ -490,6 +513,7 @@ static void put_b5_char(char *b5ch, phokey_t key)
 {
    shift_ins();
 
+   init_chpho_i(c_idx);
    bchcpy(chpho[c_idx].ch, b5ch);
    bchcpy(chpho[c_idx].och, b5ch);
    bchcpy(chpho[c_idx].ph1ch, b5ch);
@@ -504,12 +528,10 @@ static void put_b5_char(char *b5ch, phokey_t key)
    }
 }
 
-
-#define MAX_PHRASE_SEL_N (9)
+#define MAX_PHRASE_SEL_N 10
 
 static u_char selstr[MAX_PHRASE_SEL_N][MAX_PHRASE_LEN * CH_SZ];
 static u_char sellen[MAX_PHRASE_SEL_N];
-static int selidx[MAX_PHRASE_SEL_N];
 
 static u_short phrase_count;
 static u_short pho_count;
@@ -529,36 +551,35 @@ static gboolean chpho_eq_pho(int idx, phokey_t *phos, int len)
 static void get_sel_phrase()
 {
   int sti,edi,j;
-  phokey_t key, stk[MAX_PHRASE_LEN];
-  u_char len, mlen, stch[MAX_PHRASE_LEN * CH_SZ + 1];
+  u_char len, mlen;
+
+  phrase_count = 0;
 
   mlen=c_len-c_idx;
 
   if (mlen > MAX_PHRASE_LEN)
     mlen=MAX_PHRASE_LEN;
 
-  key=chpho[c_idx].pho;
-  j= key >> TSIN_HASH_SHIFT;
+  phokey_t pp[MAX_PHRASE_LEN + 1];
+  extract_pho(c_idx, mlen, pp);
 
-  if (j >= TSIN_HASH_N)
+  if (!tsin_seek(pp, 2, &sti, &edi))
     return;
 
-  sti=hashidx[j];
-  edi=hashidx[j+1];
-  phrase_count = 0;
+  while (sti < edi && phrase_count < phkbm.selkeyN) {
+    phokey_t stk[MAX_PHRASE_LEN];
+    usecount_t usecount;
+    u_char stch[MAX_PHRASE_LEN * CH_SZ + 1];
 
-  while (sti < edi && phrase_count < MAX_PHRASE_SEL_N) {
-    usecount_t usecount=0;
     load_tsin_entry(sti, &len, &usecount, stk, stch);
 
-    if (len > mlen) {
+    if (len > mlen || len==1) {
       sti++;
       continue;
     }
 
     if (chpho_eq_pho(c_idx, stk, len)) {
       sellen[phrase_count]=len;
-      selidx[phrase_count]=sti;
       utf8cpyN(selstr[phrase_count++], stch, len);
     }
 
@@ -731,7 +752,12 @@ static u_char scanphr(int chpho_idx, int plen, gboolean pho_incr)
   pre_selN = 0;
   int maxlen=0;
 
-  while (sti < edi && pre_selN < 10) {
+#define selNMax 30
+  PRE_SEL sel[selNMax];
+  int selN = 0;
+
+
+  while (sti < edi && selN < selNMax) {
     phokey_t mtk[MAX_PHRASE_LEN];
     u_char mtch[MAX_PHRASE_LEN*CH_SZ+1];
     char match_len;
@@ -744,9 +770,10 @@ static u_char scanphr(int chpho_idx, int plen, gboolean pho_incr)
       continue;
     }
 
-    int i;
 
-    for(i=0;i < plen;i++) {
+
+    int i;
+    for(i=0; i < plen; i++) {
       if (mtk[i]!=pp[i])
         break;
     }
@@ -786,15 +813,19 @@ static u_char scanphr(int chpho_idx, int plen, gboolean pho_incr)
     if (maxlen < match_len)
       maxlen = match_len;
 
-    pre_sel[pre_selN].len = match_len;
-    pre_sel[pre_selN].phidx = sti - 1;
-    pre_sel[pre_selN].usecount = usecount;
-    utf8cpyN(pre_sel[pre_selN].str, mtch, match_len);
-    memcpy(pre_sel[pre_selN].phokey, mtk, match_len*sizeof(phokey_t));
-    pre_selN++;
+    sel[selN].len = match_len;
+    sel[selN].phidx = sti - 1;
+    sel[selN].usecount = usecount;
+    utf8cpyN(sel[selN].str, mtch, match_len);
+    memcpy(sel[selN].phokey, mtk, match_len*sizeof(phokey_t));
+    selN++;
   }
 
-  qsort(pre_sel, pre_selN, sizeof(PRE_SEL), qcmp_pre_sel_usecount);
+  qsort(sel, selN, sizeof(PRE_SEL), qcmp_pre_sel_usecount);
+
+  pre_selN = Min(selN, phkbm.selkeyN);
+
+  memcpy(pre_sel, sel, sizeof(PRE_SEL) * pre_selN);
 
   return maxlen;
 }
@@ -824,32 +855,6 @@ static void disp_pre_sel_page()
   disp_selections(ph_sta);
 }
 
-
-static void raise_phr(int c)
-{
-  int i,j,tmp;
-  FILE *fp;
-
-  if (!c)
-    return;
-
-  i=selidx[c];
-  j=selidx[0];
-  tmp=phidx[i];
-  phidx[i]=phidx[j];
-  phidx[j]=tmp;
-  selidx[0]=tmp;
-
-  if ((fp=fopen(tsidxfname,"r+"))==NULL) {
-    dbg("%s modify err", tsidxfname);
-    return;
-  }
-
-  fseek(fp,sizeof(phcount)+sizeof(hashidx) + j * sizeof(int), SEEK_SET);
-  fwrite(&phidx[j],4,(c+1),fp);
-  fclose(fp);
-}
-
 void hide_selections_win();
 static void close_selection_win()
 {
@@ -858,8 +863,6 @@ static void close_selection_win()
   pre_selN = 0;
 }
 
-
-
 void show_button_pho(gboolean bshow);
 
 void tsin_set_eng_ch(int nmod)
@@ -983,6 +986,8 @@ gboolean add_to_tsin_buf(char *str, phokey_t *pho, int len)
     disp_in_area_pho_tsin();
 
     prbuf();
+
+    set_fixed(c_idx, len);
 #if 1
     for(i=1;i < len; i++) {
       chpho[c_idx+i].psta= c_idx;
@@ -1019,12 +1024,6 @@ static void set_phrase_link(int idx, int len)
 }
 #endif
 
-static void set_fixed(int idx, int len)
-{
-  int i;
-  for(i=idx; i < idx+len; i++)
-    chpho[i].flag |= FLAG_CHPHO_FIXED;
-}
 
 // should be used only if it is a real phrase
 gboolean add_to_tsin_buf_phsta(char *str, phokey_t *pho, int len)
@@ -1040,13 +1039,24 @@ gboolean add_to_tsin_buf_phsta(char *str, phokey_t *pho, int len)
     if (idx + len >= MAX_PH_BF_EXT)
       flush_tsin_buffer();
 
+    if (c_idx < c_len) {
+      int avlen = c_idx - ph_sta;
+
+      dbg("avlen:%d %d\n", avlen, len);
+      if (avlen < len) {
+        int d = len - avlen;
+
+        memmove(&chpho[c_idx + d], &chpho[c_idx], sizeof(CHPHO) * (c_len - c_idx));
+        c_len += d;
+      }
+    } else
+      c_len = idx + len;
+
     ch_pho_cpy(&chpho[idx], str, pho, len);
     set_chpho_ch2(&chpho[idx], str, len);
-
-
     set_fixed(idx, len);
 
-    c_len=c_idx=idx + len;
+    c_idx=idx + len;
 
     clrin_pho_tsin();
     disp_in_area_pho_tsin();
@@ -1116,23 +1126,9 @@ static gboolean pre_sel_handler(KeySym xkey)
   if (c >= pre_selN)
     return 0;
 
-  int j, eqlenN=0, current_ph_idx;
-
-  for(j=0; j < pre_selN; j++) {
-    if (pre_sel[j].len != len || phokey_t_seq(pre_sel[j].phokey, pre_sel[c].phokey, len))
-      continue;
-
-    if (j==c)
-      current_ph_idx = eqlenN;
-
-    selidx[eqlenN++]=pre_sel[j].phidx;
-  }
-
-//    dbg("eqlenN:%d %d\n", eqlenN, current_ph_idx);
-
-  if (eqlenN > 1) {
-    raise_phr(current_ph_idx);
-  }
+#if 0
+    dbg("eqlenN:%d %d\n", eqlenN, current_ph_idx);
+#endif
 
   full_match = FALSE;
   gboolean b_added = add_to_tsin_buf_phsta(pre_sel[c].str, pre_sel[c].phokey, len);
@@ -1187,6 +1183,14 @@ gint64 current_time()
   return (gint64)tval.tv_sec * 1000000 + tval.tv_usec;
 }
 
+static void call_tsin_parse()
+{
+  TSIN_PARSE parse[MAX_PH_BF_EXT+1];
+  bzero(parse, sizeof(parse));
+  tsin_parse(parse);
+  prbuf();
+}
+
 
 int feedkey_pp(KeySym xkey, int kbstate)
 {
@@ -1517,7 +1521,9 @@ other_keys:
          if (len > 1) {
            int cpsta = chpho[c_idx].psta;
            if (cpsta >= 0) {
+#if 0
              chpho[cpsta].flag |= FLAG_CHPHO_PHRASE_VOID;
+#endif
              set_chpho_ch(&chpho[c_idx], sel_text, len);
            } else
              set_chpho_ch2(&chpho[c_idx], sel_text, len);
@@ -1528,7 +1534,8 @@ other_keys:
 #endif
            set_fixed(c_idx, len);
 
-           raise_phr(c);
+           call_tsin_parse();
+
            if (c_idx + len == c_len) {
              ph_sta = -1;
 
@@ -1544,10 +1551,15 @@ other_keys:
 
            if (i && chpho[i].psta == i-1 && !(chpho[i-1].flag & FLAG_CHPHO_FIXED)) {
              set_chpho_ch(&chpho[i-1], chpho[i-1].ph1ch, 1);
+#if 0
              chpho[i-1].flag |= FLAG_CHPHO_PHRASE_VOID;
+#endif
            }
 
            chpho[i].flag |= FLAG_CHPHO_FIXED;
+#if 1
+           call_tsin_parse();
+#endif
          }
 
          if (len) {
@@ -1638,6 +1650,7 @@ asc_char:
           chpho[c_idx].ch[0]=tt;
         }
 
+        set_fixed(c_idx, 1);
         phokey_t tphokeys[32];
         tphokeys[0]=0;
         utf8_pho_keys(chpho[c_idx].ch, tphokeys);
@@ -1648,8 +1661,6 @@ asc_char:
         if (c_idx < c_len)
           prbuf();
 
-        orig_disp_in_area_x = -1;
-
         if (gcin_pop_up_win)
           show_win0();
 
@@ -1741,10 +1752,7 @@ llll2:
 
    put_b5_char(ch_pho[start_idx].ch, key);
 
-   TSIN_PARSE parse[MAX_PH_BF_EXT+1];
-   bzero(parse, sizeof(parse));
-   tsin_parse(parse);
-   prbuf();
+   call_tsin_parse();
 
    disp_ph_sta();
    clrin_pho_tsin();
